package isac.util;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.rmi.server.RMISocketFactory;

/**
 * This class is used for reversing the RMI Server-Client behaviour.
 * Normally the architecture design of ISAC is that the WindowApplication acts as a server
 * in order to receive the callback events from the ObjectManager.
 * 
 * This is not possible in all client configurations, particularly if the client is behind a firewall
 * (even windows firewall does not allow incoming connections by default).
 * 
 * Therefore the ReverseServerSocket does not act as a server socket and waits for a connection, 
 * but establishes a connection to the remote server given by the property file.
 * On the other end the ObjectManagerRMISocketFactory creates a server socket waits for the incomming connection.
 * 
 * This is a quick hack. The problem should be actually solved by a redesign on the architecture level.
 * 
 * @see constructor for ObjectManager
 * @see isac.session.WatchdogThread
 * @see isac.util.FixedPortRMISocketFactory
 */
public class WindowApplicationRMISocketFactory extends RMISocketFactory
{
 	
 	private String server_host_;
 	private int server_port_;
 	
 	public WindowApplicationRMISocketFactory( String server_host, int server_port)
 	{
 		super();
 		
 		server_host_ = server_host;
 		server_port_ = server_port;
 	
 	}
 	
     /**
     * Creates a client socket connected to the specified host and port
     * @param host  the host name
     * @param port  the port number
     * @return a socket connected to the specified host and port.
     * @exception IOException if an I/O error occurs during socket creation
     */
     public Socket createSocket(String host, int port)
             throws IOException
     {
         System.out.println("creating socket to host : " 
                 + host + " on port " + port);
         return new Socket(host, port);
     }

     /**
     * Create a server socket on the specified port (port 0 indicates
     * an anonymous port) and writes out some debugging info
     * @param port  the port number
     * @return  the server socket on the specified port
     * @exception IOException  if an I/O error occurs during server socket
     * creation
     */
     public ServerSocket createServerSocket(int port)
             throws IOException
     {
             System.out.println("creating ServerSocket on port " + server_port_);
             return new ReverseServerSocket(server_host_, server_port_);
     }
}

